@@ -464,9 +464,11 @@ class Parser(object):
         else:
             raise "Syntax error!"
 
-    def get(self, offset):
+    def get(self, offset, get_line = False):
         if self.pos + offset >= len(self.tokens):
             return ["", ""]
+        if get_line:
+            return self.tokens[self.pos + offset][0]
         return self.tokens[self.pos + offset][1]
     
     def get_value(self, token):
@@ -498,13 +500,42 @@ class Parser(object):
         else:
             return False
 
+    def token_type_except(self, tk, err, skip = False, i = 0):
+        if isinstance(tk, list):
+            if self.get(i)[0] in tk:
+                if skip:
+                    self.pos += 1
+            
+        if self.get(0)[0] == tk:
+            self.pos += 1
+            return
+        
+        line = self.get(i, get_line = True)
+        raise Exception("Line " + str(line) + err + "\n 你嘅类型係: " + self.get(i)[0])
+
+    def token_except(self, tk, err, skip = False, i = 0):
+        if isinstance(tk, list):
+            if self.get(i)[1] in tk:
+                if skip:
+                    self.pos += 1
+                return
+
+        elif self.get(i)[1] == tk:
+            if skip:
+                self.pos += 1
+            return
+
+        line = self.get(i, get_line = True)
+        raise Exception("Line " + str(line) + err + "\n 净系揾到: " + self.get(i)[1])
+
     # TODO: Add error check
     def parse(self):
         while True:
             if self.match(kw_print):
                 node_print_new(self.Node, self.get_value(self.get(0)))
-                self.skip(2) # Skip the args and end_print
-
+                self.skip(1) # Skip the args
+                self.token_except(kw_endprint, " : 濑嘢! 揾唔到 '点样先' ", skip = True)
+               
             elif self.match("sleep"):
                 node_sleep_new(self.Node, self.get(0))
                 self.skip(1)
@@ -518,14 +549,17 @@ class Parser(object):
                 node_let_new(self.Node, self.get_value(self.get(0)), self.get_value(self.get(2)))
                 self.skip(3)
 
-            elif self.match(kw_assign) and (self.get(1)[1] == kw_is or self.get(1)[1] == kw_is_2 or \
-                                            self.get(1)[1] == kw_is_3):
+            elif self.match(kw_assign):
+                self.token_except(tk = [kw_is, kw_is_2, kw_is_3], i = 1,  err = " : 濑嘢! 揾唔到 '係' " )
                 node_let_new(self.Node, self.get_value(self.get(0)), self.get_value(self.get(2)))
                 self.skip(3)
             
             elif self.match(kw_if):
                 cond = self.get_value(self.get(0))
-                self.skip(4) # Skip the "then", "do", "begin"
+                self.token_except(tk = kw_then, i = 1, err = " : 濑嘢! 揾唔到 '嘅话' ")
+                self.token_except(tk = kw_do, i = 2, err = " : 濑嘢! 揾唔到 '->' ")
+                self.token_except(tk = kw_begin, i = 3, err = " : 濑嘢! 揾唔到 '{' ")
+                self.skip(4) # Skip the "cond", "then", "do", "begin"
                 if_case_end = 0 # The times of case "end"
                 if_should_end = 1
                 node_if = []
@@ -552,7 +586,10 @@ class Parser(object):
             
             elif self.match(kw_elif): # case "定系" elif
                 cond = self.get_value(self.get(0))
-                self.skip(4) # Skip the "then", "do", "begin"
+                self.token_except(tk = kw_then, i = 1, err = " : 濑嘢! 揾唔到 '嘅话' ")
+                self.token_except(tk = kw_do, i = 2, err = " : 濑嘢! 揾唔到 '->' ")
+                self.token_except(tk = kw_begin, i = 3, err = " : 濑嘢! 揾唔到 '{' ")
+                self.skip(4) # Skip the "cond", "then", "do", "begin"
                 elif_case_end = 0 # The times of case "end"
                 elif_should_end = 1
                 node_elif = []
@@ -578,6 +615,9 @@ class Parser(object):
                 node_elif_new(self.Node, cond, node_elif)
 
             elif self.match(kw_else_or_not): # case "唔系" else
+                self.token_except(tk = kw_then, i = 0, err = " : 濑嘢! 揾唔到 '嘅话' ")
+                self.token_except(tk = kw_do, i = 1, err = " : 濑嘢! 揾唔到 '->' ")
+                self.token_except(tk = kw_begin, i = 2, err = " : 濑嘢! 揾唔到 '{' ")
                 self.skip(3) # Skip the "then", "do", "begin"
                 else_case_end = 0 # The times of case "end"
                 else_should_end = 1
@@ -619,6 +659,7 @@ class Parser(object):
                 if self.get(1)[0] == 'expr':
                    func_name = self.get_value(self.get(0))
                    args = self.get_value(self.get(1))
+                   self.token_except(tk = kw_func_begin, i = 2, err = " : 濑嘢! 揾唔到 '要做咩' ")
                    self.skip(3)
                    func_stmt = []
                    while self.tokens[self.pos][1][1] != kw_func_end:
@@ -628,8 +669,10 @@ class Parser(object):
                    Parser(func_stmt, node_func).parse()
                    node_func_new(self.Node, func_name, args, node_func)
                    self.skip(1) # Skip the funcend
+                
                 else:
                     func_name = self.get_value(self.get(0))
+                    self.token_except(tk = kw_func_begin, i = 1, err = " : 濑嘢! 揾唔到 '要做咩' ")
                     self.skip(2) # Skip the funcbegin
                     func_stmt = []
                     while self.tokens[self.pos][1][1] != kw_func_end:
@@ -700,6 +743,7 @@ class Parser(object):
                     func = self.get_value(self.get(0))
                     node_build_in_func_call_new(self.Node, id, func, args)
                     self.skip(2)
+
                 if self.get(0)[1] == kw_call_begin:
                     func_name = self.get_value(self.get(-1))
                     self.skip(2)
@@ -718,6 +762,8 @@ class Parser(object):
                 self.skip(1)
             
             elif self.match(kw_try):
+                self.token_except(tk = kw_do, i = 0, err = " : 濑嘢! 揾唔到 '->' ")
+                self.token_except(tk = kw_begin, i = 1, err = " : 濑嘢! 揾唔到 '{'")
                 self.skip(2) # SKip the "begin, do"
                 should_end = 1
                 case_end = 0
@@ -735,6 +781,9 @@ class Parser(object):
             
             elif self.match(kw_except):
                 _except = self.get_value(self.get(0))
+                self.token_except(tk = kw_then, i = 1, err = " : 濑嘢! 揾唔到 '嘅话'")
+                self.token_except(tk = kw_do, i = 2, err = " : 濑嘢! 揾唔到 '->'")
+                self.token_except(tk = kw_begin, i = 3, err = " : 濑嘢! 揾唔到 '{'")
                 self.skip(4) # SKip the "except", "then", "begin", "do"
                 should_end = 1
                 case_end = 0
@@ -751,6 +800,8 @@ class Parser(object):
                 node_except_new(self.Node, _except , node_except)
 
             elif self.match(kw_finally):
+                self.token_except(tk = kw_do, i = 0, err =  " : 濑嘢! 揾唔到 '->'")
+                self.token_except(tk = kw_begin, i = 1, err = " : 濑嘢! 揾唔到 '{'")
                 self.skip(2) # Skip the "begin", "do"
                 should_end = 1
                 case_end = 0
@@ -1242,7 +1293,7 @@ def cantonese_smtplib_init() -> None:
         try:
             smtpObj.sendmail(sender, receivers, message)         
             print("Successfully sent email!")
-        except SMTPException:
+        except Exception:
             print("Error: unable to send email")
     cantonese_func_def("send份邮件", send)
 
@@ -1800,17 +1851,32 @@ traditional_keywords = (
 )
 
 dump_ast = False
+dump_lex = False
+to_js = False
 
 def cantonese_run(code : str, is_to_py : bool, file : str, use_tradition : bool) -> None:
+    
     global dump_ast
+    global dump_lex
+    
     if use_tradition:
-        tokens = cantonese_token(code, tr_keywords)
+        tokens = cantonese_token(code, traditional_keywords)
     else:
         tokens = cantonese_token(code, keywords)
+    if dump_lex:
+        for token in tokens:
+            print("line " + str(token[0]) + ": " + str(token[1]))
     cantonese_parser = Parser(tokens, [])
     cantonese_parser.parse()
     if dump_ast:
         print(cantonese_parser.Node)
+    if to_js:
+        import compile
+        js, fh = compile.Compile(cantonese_parser.Node, "js", file).ret()
+        f = open(fh, 'w', encoding = 'utf-8')
+        f.write(js)
+        exit(1)
+
     run(cantonese_parser.Node, path = file)
     cantonese_lib_init()
     if is_to_py:
@@ -2290,7 +2356,7 @@ def audio(args : list, with_style : bool) -> None:
     global TO_HTML
     if len(args) == 1:
         a_beg, a_end = "<audio src = ", "</audio>\n"
-        TO_HTML += i_beg + get_str(args[0]) + i_end
+        TO_HTML += a_beg + get_str(args[0]) + a_end
 
 def sym_init() -> None:
     global sym
@@ -2476,6 +2542,224 @@ def cantonese_web_run(code : str, file_name : str, open_serv = True) -> None:
         f.write(TO_HTML)
     exit(0)
 
+class AsmLerxer(lexer):
+    def __init__(self, code, keywords, ins):
+        super().__init__(code, keywords)
+        self.ins = ins
+        self.op, self.op_get_code, self.op_gen_code, \
+        self.build_in_funcs, self.re_callfunc, self.bif_get_code, \
+        self.bif_gen_code = "", "", "", "", "", "", ""
+        self.re_register = r"[(](.*?)[)]"
+
+    def scan_register(self):
+        return self.scan(self.re_register)
+
+    def get_token(self):
+        # TODO
+        self.skip_space()
+        if len(self.code) == 0:
+            return [self.line, ['EOF', 'EOF']]
+        
+        c = self.code[0]
+
+        if c == '|':
+           token = self.scan_expr()
+           return [self.line, ['expr', token]]
+
+        if c == '(':
+            token = self.scan_register()
+            return [self.line, ['register', token]]
+        
+        if c == '{':
+            self.next(1)
+            return [self.line, ['keyword', '{']]
+        
+        if c == '}':
+            self.next(1)
+            return [self.line, ['keyword', '}']]            
+
+        if self.isChinese(c) or c == '_' or c.isalpha():
+            token = self.scan_identifier()
+            if token in self.keywords:
+                return [self.line, ['keywords', token]]
+            if token in self.ins:
+                return [self.line, ['ins', token]]
+            return [self.line, ['identifier', token]]
+        
+        if c in ('\'', '"'):
+            return [self.line, ['string', self.scan_short_string()]]
+        
+        if c == '.' or c.isdigit():
+            token = self.scan_number()
+            return [self.line, ['num', token]]
+
+        if c == '-':
+            if self.check('->'):
+                self.next(2)
+                return [self.line, ['keyword', '->']]
+        
+        self.error("睇唔明嘅Token: " + c)
+
+TO_ASM = ""
+
+class AsmParser(object):
+    def __init__(self, tokens : list, Node : list) -> None:
+        self.tokens = tokens
+        self.pos = 0
+        self.Node = Node
+
+    def get(self, offset : int) -> list:
+        if self.pos + offset >= len(self.tokens):
+            return [None, ["", ""]]
+        return self.tokens[self.pos + offset]
+        
+    def match(self, name : str) -> bool:
+        if self.get(0)[1][1] == name:
+            return True
+        return False
+    
+    def match_type(self, name : str) -> bool:
+        if self.get(0)[1][0] == name:
+            return True
+        return False
+    
+    def check(self, a, b) -> None:
+        if a == b:
+            return
+        raise LookupError("Error Token:" + str(b))
+
+    def skip(self, offset) -> None:
+            self.pos += offset
+
+    def get_value(self, token):
+        if token[1][0] == 'expr':
+            # If is expr, Remove the "|"
+            token[1][1] = token[1][1][1 : -1]
+        if token[1][0] == 'register':
+            token[1][1] = token[1][1][1 : -1]
+        return token[1][1]
+
+    def run(self, Nodes) -> None:
+        global TO_ASM
+        for node in Nodes:
+            if node[0] == 'node_data':
+                TO_ASM += "section .data\n"
+                for d in node[1]:
+                    TO_ASM += d[0] + " equ " + d[1] + "\n"
+
+            if node[0] == 'node_mov':
+                TO_ASM += "mov " + node[1][0] + ", " + node[1][1] + "\n"
+
+            if node[0] == 'node_int':
+                TO_ASM += "int " + node[1][0] + "\n"
+
+            if node[0] == 'node_global':
+                TO_ASM += "global " + node[1][0] + "\n"
+
+            if node[0] == 'node_code':
+                TO_ASM += node[1][0] + ":\n"
+                self.run(node[1][1])
+
+    def parse(self) -> None:
+        while True:
+            if self.match_type('expr'):
+                if self.get(1)[1][1] == "有咩":
+                    data_name = self.get(0)[1]
+                    self.skip(4)
+                    data_stmt = []
+                    while self.tokens[self.pos][1][1] != "}":
+                        data_stmt.append(self.tokens[self.pos])
+                        self.pos += 1
+                    self.skip(1)
+                    ret = self.data_parse(data_stmt, data_name)
+                    asm_ast_new(self.Node, "node_data", ret)
+            
+                if self.get(1)[1][1] == "要做咩":
+                    code_name = self.get_value(self.get(0))
+                    self.skip(4)
+                    code_stmt = []
+                    while self.tokens[self.pos][1][1] != "}":
+                        code_stmt.append(self.tokens[self.pos])
+                        self.pos += 1
+                    self.skip(1)
+                    stmt_node = []
+                    AsmParser(code_stmt, stmt_node).parse()
+                    asm_ast_new(self.Node, "node_code", [code_name, stmt_node])
+
+                if self.get(1)[1][1] == "排头位":
+                    start_name = self.get_value(self.get(0))
+                    self.skip(4)
+                    asm_ast_new(self.Node, "node_global", [start_name])
+
+            elif self.match_type('register'):
+                reg = self.get_value(self.get(0))
+                self.skip(1)
+                if self.get(0)[1][1] == "收数":
+                    self.skip(2)
+                    data = self.get_value(self.get(0))
+                    asm_ast_new(self.Node, 'node_mov', [reg, data])
+                    self.skip(1)
+            
+            elif self.match("仲要等埋"):
+                self.skip(2)
+                addr = self.get_value(self.get(0))
+                asm_ast_new(self.Node, 'node_int', [addr])
+                self.skip(1)
+
+            else:
+                break
+
+    def data_parse(self, stmt, name) -> None:
+        return dataParser(stmt, []).parse(name)
+        
+class dataParser(AsmParser):
+    def parse(self, name) -> None:
+        ret = []
+        while True:
+            if self.match_type("identifier"):
+                key = self.get_value(self.get(0))
+                self.skip(1)
+                self.check(self.get(0)[1][1], "係")
+                self.skip(1)
+                val = self.get_value(self.get(0))
+                self.skip(1)
+                ret.append([key, val])
+            else:
+                break
+        return ret
+
+def asm_ast_new(Node : list, type : str, ctx : list) -> None:
+    Node.append([type, ctx])
+
+
+def Cantonese_asm_run(code : str, file_name : str) -> None:
+    global TO_ASM
+
+    ins_mov_from = "收数"
+    ins_int = "仲要等埋"
+    kw_datadef = "有咩"
+    kw_secdef = "要做咩"
+    kw_is = "係"
+    kw_main = "排头位"
+    kw_begin = "开工"
+
+    keywords = (kw_datadef, kw_secdef,
+            kw_is, kw_main, kw_begin)
+    ins = (ins_mov_from, ins_int)
+    lex = AsmLerxer(code, keywords, ins)
+    tokens = []
+
+    while True:
+        token = lex.get_token()
+        tokens.append(token)
+        if token[1] == ['EOF', 'EOF']:
+            break
+
+    AST = []
+    AsmParser(tokens, AST).parse()
+    AsmParser(tokens, AST).run(AST)
+    print(TO_ASM)
+    exit(1)
 
 class 交互(cmd.Cmd):
     def __init__(self):
@@ -2494,6 +2778,8 @@ def main():
     arg_parser = argparse.ArgumentParser()
     arg_parser.add_argument("file", nargs = '?', default = "")
     arg_parser.add_argument("-to_py", action = "store_true")
+    arg_parser.add_argument("-to_js", action = "store_true")
+    arg_parser.add_argument("-to_asm", action = "store_true")
     arg_parser.add_argument("-讲翻py", action = "store_true")
     arg_parser.add_argument("-to_web", action = "store_true")
     arg_parser.add_argument("-倾偈", action = "store_true")
@@ -2503,11 +2789,14 @@ def main():
     arg_parser.add_argument("-install", action = "store_true")
     arg_parser.add_argument("-stack_vm", action = "store_true")
     arg_parser.add_argument("-ast", action = "store_true")
+    arg_parser.add_argument("-lex", action = "store_true")
     arg_parser.add_argument("-debug", action = "store_true")
     args = arg_parser.parse_args()
 
     global use_tradition
     global dump_ast
+    global dump_lex
+    global to_js
     global debug
 
     if not args.file:
@@ -2537,11 +2826,17 @@ def main():
                     cantonese_web_run(code, args.file, True)
             if args.ast:
                 dump_ast = True
+            if args.lex:
+                dump_lex = True
             if args.debug:
                 debug = True
             if args.stack_vm:
                 cantonese_run_with_vm(code, args.file, use_tradition)
                 exit(1)
+            if args.to_js:
+                to_js = True
+            if args.to_asm:
+                Cantonese_asm_run(code, args.file)
             cantonese_run(code, is_to_py, args.file, use_tradition)
     except FileNotFoundError:
         print("揾唔到你嘅文件 :(")
